using System;
using System.Configuration;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Obrancovia
{
    public class MainUiScreen : UiScreen, IDisposable
        
        #region Premenne triedy
        private Texture texturyTlacitka = null;
        private Texture texturaPozadi = null;

        private UiButton tlacitkoNovaHra = null;
        private UiButton tlacitkoKoniec = null;

        // Udalosti
        public event EventHandler NovaHra;
        public event EventHandler Koniec;

        #endregion

        #region Vytvorenie
        /// <summary>
        /// Vytvor obrazovku s hlavnm menu.
        /// </summary>
        public MainUiScreen(Device zariadenie, int sirka, int vyska)
            : base(zariadenie, sirka, vyska)
        {
            // Vytvor textru pre pozadie.
            texturaPozadia = TextureLoader.FromFile(zariadenie, HerneJadro.MediaPath
              + "HlavneMenu.png");

            // Textru pozadia chcem zobrazi cez cel obrazovku.
            UlozTexturu(texturaPozadia, sirka, vyska, false);

             // Vytvor textry pre tlatka
            texturyTlacitka = TextureLoader.FromFile(zariadenie, HerneJadro.MediaPath
              + "HlavneTlacitka.png");


            // Vytvor tlatka hlavnho menu.

            // Vytvor tlatko novej hry.
            tlacitkoNovaHra = new UiButton(zobrazSprite, texturyTlacitka,
              texturyTlacitka,
              new Rectangle(0, VyskaVelkehoTlacitka * 1,
              SirkaVelkehoTlacitka, VyskaVelkehoTlacitka),
              new Rectangle(0, 0,
              SirkaVelkehoTlacitka, VyskaVelkehoTlacitka),
              new Point((sirka - SirkaVelkehoTlacitka) / 2,
              (vyska - (VyskaVelkehoTlacitka * 4)) / 2));

            tlacitkoNovaHra.Click += new EventHandler(OnNewButton);

            // Vytvor ukonovacie tlatko.
            tlacitkoKoniec = new UiButton(zobrazSprite, texturyTlacitka,
              texturyTlacitka, new Rectangle(0, VyskaVelkehoTlacitka * 3,
              SirkaVelkehoTlacitka, VyskaVelkehoTlacitka),
              new Rectangle(0, VyskaVelkehoTlacitka * 2,
              SirkaVelkehoTlacitka, VyskaVelkehoTlacitka),
              new Point((sirka - SirkaVelkehoTlacitka) / 2,
              (vyska - (VyskaVelkehoTlacitka * 2)) / 2));

            tlacitkoKoniec.Click += new EventHandler(OnExitButton);
        }

        #endregion

        #region Zobrazenie
        /// <summary>
        /// Zobraz obrazovku s hlavnm menu.
        /// </summary>
        public override void Draw()
        {
            // Zana vykresova sprity.
            base.ZaciatokSprite();
            // Kresl pozadie.
            base.Draw();
            // Kresl tlatka.
            tlacitkoNovaHra.Draw();
            tlacitkoKoniec.Draw();
            // Kon kreslenie spritov.
            base.KoniecSprite();
        }

        #endregion

        #region Obsluha udalosti kliknutia na tlacitka
        /// <summary>
        /// Spusten, ke kliknem na tlatko.
        /// </summary>
        private void OnNewButton(object sender, EventArgs e)
        {
            if (NovaHra != null)
                NovaHra(this, e);
        }

        
        /// <summary>
        /// Spusten, ke klikneme na tlatko.
        /// </summary>
        private void OnExitButton(object sender, EventArgs e)
        {
            if (Koniec != null)
                Koniec(this, e);
        }
        #endregion

        #region Udalosti klavesnice a mysi
        /// <summary>
        /// Aktualizuje stav tlatiek, ak sa nad nimi nachdza my.
        /// </summary>
        public void OnMouseMove(int x, int y)
        {
            tlacitkoNovaHra.OnMouseMove(x, y);
            tlacitkoKoniec.OnMouseMove(x, y);
        }

        /// <summary>
        /// Kontroluje, i uvate klikol na niektor tlatko.
        /// </summary>
        public void OnMouseClick(int x, int y)
        {
            tlacitkoNovaHra.OnMouseClick(x, y);
            tlacitkoKoniec.OnMouseClick(x, y);
        }

        
        /// <summary>
        /// Kontroluje, i uvate stisol klvesu na klvesnici.
        /// </summary>
        public void OnKeyPress(System.Windows.Forms.Keys klavesa)
        {
            switch (klavesa)
            {
                // Stisnutie klvesy Esc= kliknutie na ukonovacie tlatko.
                case Keys.Escape:
                    OnExitButton(this, EventArgs.Empty);
                    break;
                 // Stisnutie klvesy Enter= kliknutie na tlatko novej hry.
                case Keys.Enter:
                    OnNewButton(this, EventArgs.Empty);
                    break;
            }
        }
        #endregion

        #region IDisposable Members
        /// <summary>
        /// Upratovanie v prpade, e Dispose nebolo volan.
        /// </summary>
        ~MainUiScreen()
        {
            Dispose();
        }
        /// <summary>
        /// Upratovanie zdrojov.
        /// </summary>
        public override void Dispose()
        {
            GC.SuppressFinalize(this);
            if (texturaPozadia != null)
            {
                texturaPozadia.Dispose();
            }
            if (texturyTlacitka != null)
            {
                texturyTlacitka.Dispose();
            }
            texturyTlacitka = null;
            texturaPozadia = null;
            base.Dispose();
        }

        #endregion
    }

    /// <summary>
    /// Obrazovka s vberom hernej postavy (Loopy).
    /// </summary>
    public class SelectLoopyScreen : UiScreen, IDisposable
    {
        #region Atribty triedy
        // Kontanty
        private static readonly Vector3 umiestnenieKamery = new Vector3(
          0, 20.0f, -85.0f);
        private static readonly Vector3 cielKamery = new Vector3(0, 20, 0);
        private static readonly Vector3 kameraHore = new Vector3(0, 1.0f, 0);

        private Texture texturyTlacitka = null;
        private Texture texturaPozadi = null;
        private UiButton tlacitkoOk = null;
        private UiButton tlacitkoDolava = null;
        private UiButton tlacitkoDoprava = null;

        private Mesh meshLoopyho = null;
        private Texture texturaLoopyho = null;
        private Material materialLoopyho;
        private Device ulozeneZariadenie = null;
        private FarbaPostavy farbaLoopyho = FarbaPostavy.Min;

        // Udalosti
        public event EventHandler Zvolena;
        #endregion

        #region Vytvorenie
        /// <summary>
        /// Vytvor obrazovky na vber hernej postavy
        /// </summary>
        /// <param name="device"></param>
        public SelectLoopyScreen(Device zariadenie, Texture tlacitka, int sirka,
            int vyska)
            : base(zariadenie, sirka, vyska)
        {
            // Ulo zariadenie pre dan textru.
            ulozeneZariadenie = zariadenie;

            // Ulo textry pre tlatka.
            texturyTlacitka = tlacitka;

            // Vytvor sie Loopyho.
            meshLoopyho = Mesh.FromFile(HerneJadro.MediaPath + "Loopy.x",
              MeshFlags.Managed, zariadenie);

            // Vytvor textru pre Loopyho.
            VytvorTexturu(zariadenie);

            // Vytvor materil.
            materialLoopyho = new Material();
            materialLoopyho.Diffuse = materialLoopyho.Ambient = Color.White;

            // Vytvor textru pozadia.
            texturaPozadia = TextureLoader.FromFile(zariadenie, HerneJadro.MediaPath +
              "Vyber.png");
            UlozTexturu(texturaPozadia, sirka, vyska / 2, true);

            // Vytvor potvrdzovacie tlatko.
            tlacitkoOk = new UiButton(zobrazSprite, texturyTlacitka, texturyTlacitka,
              new Rectangle(SirkaMalehoTlacitka, VyskaMalehoTlacitka * 3,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Rectangle(0, VyskaMalehoTlacitka * 3,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Point((sirka - SirkaMalehoTlacitka) / 2,
              vyska - (VyskaMalehoTlacitka * 2)));

            tlacitkoOk.Click += new EventHandler(OnSelectButton);

            // Vytvor tlatka pre posun dolava a doprava.
            tlacitkoDolava = new UiButton(zobrazSprite, texturyTlacitka,
              texturyTlacitka,
              new Rectangle(SirkaMalehoTlacitka, 0, SirkaMalehoTlacitka,
              VyskaMalehoTlacitka),
              new Rectangle(0, 0, SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Point(SirkaMalehoTlacitka / 2,
              vyska - (VyskaMalehoTlacitka * 2)));

            tlacitkoDoleva.Click += new EventHandler(OnLeftButton);

            tlacitkoDoprava = new UiButton(zobrazSprite, texturyTlacitka,
              texturyTlacitka,
              new Rectangle(SirkaMalehoTlacitka, VyskaMalehoTlacitka * 1,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Rectangle(0, VyskaMalehoTlacitka * 1,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Point(sirka - (SirkaMalehoTlacitka + (SirkaMalehoTlacitka / 2)),
              vyska - (VyskaMalehoTlacitka * 2)));

            tlacitkoDoprava.Click += new EventHandler(OnRightButton);

            UlozTexturu(texturaPozadia, sirka, vyska / 2, true);
        }

        /// <summary>
        /// Vytvor textru Loopyho.
        /// </summary>
        private void VytvorTexturu(Device zariadenie)
        {
            if (texturaLoopyho != null)
            {
                // Uvon textru a vytvor nov.
                texturaLoopyho.Dispose();
            }
            texturaLoopyho = TextureLoader.FromFile(zariadenie,
              ZiskajNazovSuboruPodlaFarby(farbaLoopyho));
        }

        /// <summary>
        /// Vracia nzov sboru s textrou pre farbu hernej postavy.
        /// </summary>
        private static string ZiskajNazovSuboruPodlaFarby(FarbaPostavy farba)
        {
            switch (farba)
            {
                case FarbaPostavy.Modra:
                    return HerneJadro.MediaPath + "LoopyModry.bmp";
                case FarbaPostavy.Zelena:
                    return HerneJadro.MediaPath + "LoopyZeleny.bmp";
                case FarbaPostavy.Fialova:
                    return HerneJadro.MediaPath + "LoopyFialovy.bmp";
                case FarbaPostavy.Cervena:
                    return HerneJadro.MediaPath + "LoopyCerveny.bmp";
                case FarbaPostavy.Zlta:
                    return HerneJadro.MediaPath + "LoopyZlty.bmp";
                case FarbaPostavy.Seda:
                    return HerneJadro.MediaPath + "LoopySedy.bmp";
                default:
                    throw new ArgumentException("farba",
                      "Neexistujica farba ako parameter.");
            }
        }

        #endregion

        #region Zobrazenie
        /// <summary>
        /// Zobraz tlatka.
        /// </summary>
        public void Draw(float celkovyCas)
        {
            // Zobraz Loopyho. Nastav transformciu sveta.
            ulozeneZariadenie.Transform.View = Matrix.LookAtLH(
              umiestnenieKamery, cielKamery, kameraHore);

            ulozeneZariadenie.Transform.World = Matrix.RotationY(celkovyCas * 2);

            ulozeneZariadenie.SetTexture(0, texturaLoopyho);
            ulozeneZariadenie.Material = materialLoopyho;
            meshLoopyho.DrawSubset(0);

            // Zana kresli obrzky.
            base.ZaciatokSprite();
            // Vykresl pozadie.
            base.Draw();
            // Teraz tlatka.
            tlacitkoDolava.Draw();
            tlacitkoDoprava.Draw();
            tlacitkoOk.Draw();

            // Obrzky s vykreslen.
            base.KoniecSprite();
        }

        #endregion

        #region Upratovanie

        /// <summary>
        /// Uprace sie a textru hernej postavy.
        /// </summary>
        public void UvolniMeshLoopyho()
        {
            if (meshLoopyho != null)
            {
                meshLoopyho.Dispose();
            }
            if (texturaLoopyho != null)
            {
                texturaLoopyho.Dispose();
            }
        }

        /// <summary>
        /// Upratovanie v prpade, e Dispose nebolo volan.
        /// </summary>
        ~SelectLoopyScreen()
        {
            Dispose();
        }
        /// <summary>
        /// Upratovanie zdrojov.
        /// </summary>
        public override void Dispose()
        {
            GC.SuppressFinalize(this);
            if (texturaPozadi != null)
            {
                texturaPozadi.Dispose();
            }
            texturaPozadi = null;
            base.Dispose();
        }
        #endregion

        #region Vlastnosti
        /// <summary>
        /// Vrti sie hernej postavy.
        /// </summary>
        public Mesh MeshLoopyho
        {
            get { return meshLoopyho; }
        }
        /// <summary>
        /// Vrti textru hernej postavy.
        /// </summary>
        public Texture TexturaLoopyho
        {
            get { return texturaLoopyho; }
        }
        /// <summary>
        /// Vrti materil hernej postavy.
        /// </summary>
        public Material MaterialLoopyho
        {
            get { return materialLoopyho; }
        }
        #endregion

        #region Obsluha udalosti kliknutia na tlacitko.
        /// <summary>
        /// Spusten, ke klinem na tlatko pre posun doava.
        /// </summary>
        private void OnLeftButton(object sender, EventArgs e)
        {
            farbaLoopyho--;
            if (farbaLoopyho < FarbaPostavy.Min)
                farbaLoopyho = FarbaPostavy.Max;
            // Vytvor prslun textru.
            VytvorTexturu(ulozeneZariadenie);
        }
        /// <summary>
        /// Spusten, ke kliknem na tlatko pre posun doprava.
        /// </summary>
        private void OnRightButton(object sender, EventArgs e)
        {
            farbaLoopyho++;
            if (farbaLoopyho > FarbaPostavy.Max)
                farbaLoopyho = FarbaPostavy.Min;
            // Vytvor prslun textru.
            VytvorTexturu(ulozeneZariadenie);
        }
        /// <summary>
        /// Spusten, ke klikneme na potvrdzovacie tlatko.
        /// </summary>
        private void OnSelectButton(object sender, EventArgs e)
        {
            if (Zvolena != null)
                Zvolena(this, e);
        }
        #endregion

        #region Udalosti klavesnice a mysi
        public void OnMouseMove(int x, int y)
        {
            tlacitkoDolava.OnMouseMove(x, y);
            tlacitkoDoprava.OnMouseMove(x, y);
            tlacitkoOk.OnMouseMove(x, y);
        }
        public void OnMouseClick(int x, int y)
        {
            tlacitkoDolava.OnMouseClick(x, y);
            tlacitkoDoprava.OnMouseClick(x, y);
            tlacitkoOk.OnMouseClick(x, y);
        }
        public void OnKeyPress(System.Windows.Forms.Keys klavesa)
        {
            switch (klavesa)
            {
                case Keys.Enter:
                    // Stisnutie klvesy Enter zodpoved vberu hernej postavy.
                    OnSelectButton(this, EventArgs.Empty);
                    break;
                case Keys.Left:
                    // Stisnutie klvesy pka dolava zodpoved kliknutiu na tlatko pre
                    // posun dolava.
                    OnLeftButton(this, EventArgs.Empty);
                    break;
                case Keys.Right:
                    OnRightButton(this, EventArgs.Empty);
                    break;
            }
        }
        #endregion
    }
    /// <summary>
    /// Ukonovacia obrazovka
    /// </summary>
    public class QuitScreen : UiScreen, IDisposable
    {
        #region Premenne triedy
        private Texture texturyTlacitka = null;
        private Texture texturaPozadia = null;
        private UiButton tlacitkoAno = null;
        private UiButton tlacitkoNie = null;

        // Udalosti
        public event EventHandler QuitGame;
        public event EventHandler Cancel;
        #endregion

        #region Kontruktor
        /// <summary>
        /// Kontruktor objektov triedy QuitScreen.
        /// </summary>
        public QuitScreen(Device zariadenie, Texture tlacitka, int sirka,
          int vyska)
            : base(zariadenie, sirka, vyska)
        {
            // Vytvorme textry pre tlatka.
            texturyTlacitka = tlacitka;

            // Vytvorme textru pozadia.
            texturaPozadia = TextureLoader.FromFile(zariadenie, HerneJadro.MediaPath
              + "PotvrdenieUkoncenia.png");
            UlozTexturu(texturaPozadia, sirka, vyska, true);

            // Vypota rozostup pre tlatka.
            float vodorovnyRozostup = (float)(sirka - (SirkaMalehoTlacitka * 3))
              / 2.0f;

            // Vytvorme tlatka no/nie.
            tlacitkoAno = new UiButton(zobrazSprite, texturyTlacitka,
              texturyTlacitka,
              new Rectangle(SirkaMalehoTlacitka, VyskaMalehoTlacitka * 3,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Rectangle(0, VyskaMalehoTlacitka * 3,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Point((int)vodorovnyRozostup,
              (int)(centerUpper.Y + zdrojPozadia.Height)
              - (VyskaMalehoTlacitka * 2)));

            tlacitkoAno.Click += new EventHandler(OnYesButton);

            tlacitkoNe = new UiButton(zobrazSprite, texturyTlacitka,
              texturyTlacitka,
              new Rectangle(SirkaMalehoTlacitka, VyskaMalehoTlacitka * 2,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Rectangle(0, VyskaMalehoTlacitka * 2,
              SirkaMalehoTlacitka, VyskaMalehoTlacitka),
              new Point(sirka - ((int)vodorovnyRozostup + SirkaMalehoTlacitka),
              (int)(centerUpper.Y + zdrojPozadia.Height)
              - (VyskaMalehoTlacitka * 2)));

            tlacitkoNe.Click += new EventHandler(OnNoButton);
        }

        #endregion

        #region Zobrazenie
        /// <summary>
        /// Zobraz tlatka.
        /// </summary>
        public override void Draw()
        {
            // Zana zobrazova.
            base.ZaciatokSprite();
            // Vykresl pozadie.
            base.Draw();
            // Teraz vykresl tlatka.
            tlacitkoAno.Draw();
            tlacitkoNie.Draw();

            // Koniec zobrazovania.
            base.KoniecSprite();
        }

        #endregion

        #region IDisposable Members
        /// <summary>
        /// Upratovanie v prpade, e Dispose nebolo volan.
        /// </summary>
        ~QuitScreen()
        {
            Dispose();
        }
        /// <summary>
        /// Uprace vytvoren zdroje.
        /// </summary>
        public override void Dispose()
        {
            GC.SuppressFinalize(this);
            if (texturaPozadia != null)
            {
                texturaPozadia.Dispose();
            }
            texturaPozadia = null;
            texturyTlacitka = null;
            base.Dispose();
        }

        #endregion

        #region Obsluha udalosti klepnuti na tlacitka
        /// <summary>
        /// Spusten, ke hr klikne na tlatko potvrdenie konca.
        /// </summary>
        private void OnYesButton(object sender, EventArgs e)
        {
            if (QuitGame != null)
                QuitGame(this, EventArgs.Empty);
        }
        
        /// <summary>
        /// Spusten, ke hr klikne na tlatko zamietnutie konca.
        /// </summary>
        private void OnNoButton(object sender, EventArgs e)
        {
            if (Cancel != null)
                Cancel(this, EventArgs.Empty);
        }
        #endregion

        #region Udalosti klavesnice a mysi
        /// <summary>
        /// Zist, i sa ukazovate myi pohybuje nad tlatkami.
        /// </summary>
        public void OnMouseMove(int x, int y)
        {
            tlacitkoAno.OnMouseMove(x, y);
            tlacitkoNie.OnMouseMove(x, y);
        }

        /// <summary>
        /// Zist, i hr klikol na tlatko.
        /// </summary>
        public void OnMouseClick(int x, int y)
        {
            tlacitkoAno.OnMouseClick(x, y);
            tlacitkoNie.OnMouseClick(x, y);
        }

        /// <summary>
        /// Volan, ke hr stiskne klvesu.
        /// </summary>
        public void OnKeyPress(System.Windows.Forms.Keys klavesa)
        {
            switch (klavesa)
            {
                case Keys.Escape:
                case Keys.N:
                    // Stisnutie klvesy Esc zodpoved kliknutiu na tlatko
                    // s odmietnutm.
                    OnNoButton(this, EventArgs.Empty);
                    break;
                case Keys.Enter:
                case Keys.Y:
                    // Stisnutie klvesy Enter zodpoved kliknutiu na potvrdzovacie
                    // tlatko.
                    OnYesButton(this, EventArgs.Empty);
                    break;
            }
        }
        #endregion
    }
}

