using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Configuration;
using System.IO;

namespace EditorUrovni
{
    /// <summary>
    /// Mon barvy kostky.
    /// </summary>
    public enum BarvaKostky : byte
    {
        Red = 0,
        Blue = 1,
        Green = 2,
        Yellow = 3,
        SandyBrown = 4,
        WhiteSmoke = 5,
        DarkGoldenRod = 6,
        CornflowerBlue = 7,
        Pink = 8
    }

    /// <summary>
	/// Tda Form1.
	/// </summary>
	public unsafe class Form1 : System.Windows.Forms.Form
	{
		
        private static readonly Color[] BarvyKostky = {Color.Black, Color.Red , Color.Blue, Color.Green, 
                                                        Color.Yellow, Color.SandyBrown, Color.WhiteSmoke,
                                                        Color.DarkGoldenrod, Color.CornflowerBlue, Color.Pink
                                                    };

        
        private System.ComponentModel.Container components = null;
        private static readonly string MediaPath = ConfigurationSettings.AppSettings.Get("MediaPath");
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.Label label4;
        private System.Windows.Forms.CheckBox chkWrap;

        private System.Windows.Forms.Button[] tlacitkaUrovne;
        private byte[] ukazateleUrovne;
        private System.Windows.Forms.Button[] hlavniTlacitka;
        private int[] hlavniBarvy;

        private System.Windows.Forms.Label label5;
        private System.Windows.Forms.Label label6;
        private System.Windows.Forms.TextBox txtCas;
        private System.Windows.Forms.TextBox txtPohyby;
        private System.Windows.Forms.NumericUpDown poradiUrovne;
        private System.Windows.Forms.Label label7;
        private System.Windows.Forms.Button tlacitkoUloz;

        int indexKostkyPostavy = 0;

        private const int VelikostPlochy = 13;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
            InitializeComponent();
            // Vytvome hlavn tlatka.
            hlavniTlacitka = new Button[4];
            hlavniBarvy = new int[4];
            for (int i = 0; i < 4; i++)
            {
                hlavniTlacitka[i] = new Button();
                hlavniTlacitka[i].Tag = i;
                hlavniTlacitka[i].BackColor = System.Drawing.Color.Black;
                hlavniTlacitka[i].Location = new System.Drawing.Point(
                  16 + (72 * i), 40);
                hlavniTlacitka[i].Size = new System.Drawing.Size(24, 24);
                hlavniTlacitka[i].TabStop = false;
                hlavniTlacitka[i].Click += new EventHandler(OnMainButtonClick);
                hlavniBarvy[i] = 0;
                this.Controls.Add(hlavniTlacitka[i]);
            }
            tlacitkaUrovne = new Button[VelikostPlochy * VelikostPlochy];
            ukazateleUrovne = new byte[tlacitkaUrovne.Length];
            int index = 0;
            this.SuspendLayout();
            Font f = new Font(this.Font.Name, 10.0f);
            for (int j = 0; j < VelikostPlochy; j++)
            {
                for (int i = 0; i < VelikostPlochy; i++)
                {
                    tlacitkaUrovne[index] = new Button();
                    tlacitkaUrovne[index].Tag = index;
                    tlacitkaUrovne[index].Font = f;
                    tlacitkaUrovne[index].BackColor = System.Drawing.Color.Black;
                    tlacitkaUrovne[index].Location = new System.Drawing.Point(2 +
                      (22 * i), 80 + (22 * j));
                    tlacitkaUrovne[index].Size = new System.Drawing.Size(20, 20);
                    tlacitkaUrovne[index].TabStop = false;
                    tlacitkaUrovne[index].Click += new EventHandler(OnLevelButtonClick);
                    ukazateleUrovne[index] = 0;
                    this.Controls.Add(tlacitkaUrovne[index]);
                    index++;
                }
            }
            this.ResumeLayout(true);
        }

        private void AktualizujBarvyKostky()
        {
            for (int i = 0; i < tlacitkaUrovne.Length; i++)
            {
                if (ukazateleUrovne[i] < 4)
                {
                    tlacitkaUrovne[i].BackColor =
                      hlavniTlacitka[ukazateleUrovne[i]].BackColor;
                }
                else
                {
                    tlacitkaUrovne[i].BackColor = System.Drawing.Color.Black;
                }
                if (indexKostkyPostavy == i)
                {
                    if (tlacitkaUrovne[i].BackColor == System.Drawing.Color.Black)
                    {
                        indexKostkyPostavy++;
                    }
                    else
                    {
                        tlacitkaUrovne[i].Text = "*";
                    }
                }
                else
                {
                    tlacitkaUrovne[i].Text = string.Empty;
                }
            }
        }

        private void OnSaveLevel(object sender, System.EventArgs e)
        {
            FileStream stm = File.Create(MediaPath + string.Format("Uroven{0}.lvl",
              (int)poradiUrovne.Value));

            float maxCas = float.Parse(txtCas.Text);
            int maxPohyby = int.Parse(txtPohyby.Text);

            stm.Write(BitConverter.GetBytes(maxCas), 0, sizeof(float));
            stm.Write(BitConverter.GetBytes(maxPohyby), 0, sizeof(int));

            int pctBarev = 0;
            for (int i = 0; i < 4; i++)
            {
                if (hlavniBarvy[i] > 0)
                    pctBarev++;
                else
                    break;
            }
            if (pctBarev < 2)
                throw new InvalidOperationException("Musite definovat vice barev.");
            BarvaKostky[] barvy = new BarvaKostky[pctBarev];
            int pocetBarev = barvy.Length;
            stm.Write(BitConverter.GetBytes(pocetBarev), 0, sizeof(int));
            for (int i = 0; i < barvy.Length; i++)
            {
                barvy[i] = (BarvaKostky)(hlavniBarvy[i] - 1);
                stm.WriteByte((byte)barvy[i]);
            }

            BarvaKostky cilova = barvy[barvy.Length - 1];

            stm.WriteByte((byte)cilova);
            stm.Write(BitConverter.GetBytes(chkWrap.Checked), 0, sizeof(bool));

            int indexKostky = 0;
            for (int j = 0; j < VelikostPlochy; j++)
            {
                for (int i = 0; i < VelikostPlochy; i++)
                {
                    if (ukazateleUrovne[indexKostky] < pctBarev)
                    {
                        stm.WriteByte(ukazateleUrovne[indexKostky]);
                    }
                    else
                    {
                        stm.WriteByte((byte)0xff);
                    }
                    indexKostky++;
                }
            }

            stm.Write(BitConverter.GetBytes(indexKostkyPostavy), 0, 4);
            stm.Close();
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
        private void InitializeComponent()
        {
            this.label1 = new System.Windows.Forms.Label();
            this.label2 = new System.Windows.Forms.Label();
            this.label3 = new System.Windows.Forms.Label();
            this.label4 = new System.Windows.Forms.Label();
            this.chkWrap = new System.Windows.Forms.CheckBox();
            this.label5 = new System.Windows.Forms.Label();
            this.label6 = new System.Windows.Forms.Label();
            this.txtCas = new System.Windows.Forms.TextBox();
            this.txtPohyby = new System.Windows.Forms.TextBox();
            this.poradiUrovne = new System.Windows.Forms.NumericUpDown();
            this.label7 = new System.Windows.Forms.Label();
            this.tlacitkoUloz = new System.Windows.Forms.Button();
            ((System.ComponentModel.ISupportInitialize)(this.poradiUrovne)).BeginInit();
            this.SuspendLayout();

            this.label1.Location = new System.Drawing.Point(8, 8);
            this.label1.Size = new System.Drawing.Size(64, 23);
            this.label1.TabIndex = 0;
            this.label1.Text = "Barva 1";

            this.label2.Location = new System.Drawing.Point(80, 8);
            this.label2.Size = new System.Drawing.Size(64, 23);
            this.label2.TabIndex = 1;
            this.label2.Text = "Barva 2";

            this.label3.Location = new System.Drawing.Point(160, 8);
            this.label3.Size = new System.Drawing.Size(64, 23);
            this.label3.TabIndex = 2;
            this.label3.Text = "Barva 3";

            this.label4.Location = new System.Drawing.Point(232, 8);
            this.label4.Size = new System.Drawing.Size(64, 23);
            this.label4.TabIndex = 3;
            this.label4.Text = "Barva 4";

            this.chkWrap.Location = new System.Drawing.Point(296, 40);
            this.chkWrap.Size = new System.Drawing.Size(120, 16);
            this.chkWrap.TabIndex = 4;
            this.chkWrap.Text = "Maji barvy pretekat?";

            this.label5.Location = new System.Drawing.Point(296, 72);
            this.label5.Size = new System.Drawing.Size(104, 23);
            this.label5.TabIndex = 9;
            this.label5.Text = "Pocet sekund";

            this.label6.Location = new System.Drawing.Point(296, 136);
            this.label6.Size = new System.Drawing.Size(104, 23);
            this.label6.TabIndex = 10;
            this.label6.Text = "Max pohybu";

            this.txtCas.Location = new System.Drawing.Point(296, 96);
            this.txtCas.Size = new System.Drawing.Size(96, 20);
            this.txtCas.TabIndex = 11;
            this.txtCas.Text = "180.0";

            this.txtPohyby.Location = new System.Drawing.Point(296, 168);
            this.txtPohyby.Size = new System.Drawing.Size(96, 20);
            this.txtPohyby.TabIndex = 12;
            this.txtPohyby.Text = "500";

            this.poradiUrovne.Location = new System.Drawing.Point(296, 232);
            this.poradiUrovne.Minimum = new System.Decimal(new int[] {
    1, 0, 0, 0});
            this.poradiUrovne.Size = new System.Drawing.Size(104, 20);
            this.poradiUrovne.TabIndex = 13;
            this.poradiUrovne.Value = new System.Decimal(new int[] {
    1, 0, 0, 0});

            this.label7.Location = new System.Drawing.Point(296, 208);
            this.label7.Size = new System.Drawing.Size(104, 23);
            this.label7.TabIndex = 14;
            this.label7.Text = "Poradi urovne";

            this.tlacitkoUloz.Location = new System.Drawing.Point(304, 320);
            this.tlacitkoUloz.Size = new System.Drawing.Size(104, 24);
            this.tlacitkoUloz.TabIndex = 15;
            this.tlacitkoUloz.Text = "Ulozit uroven";
            this.tlacitkoUloz.Click += new System.EventHandler(this.OnSaveLevel);

            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(424, 382);
            this.Controls.Add(this.tlacitkoUloz);
            this.Controls.Add(this.label7);
            this.Controls.Add(this.poradiUrovne);
            this.Controls.Add(this.txtPohyby);
            this.Controls.Add(this.txtCas);
            this.Controls.Add(this.label6);
            this.Controls.Add(this.label5);
            this.Controls.Add(this.chkWrap);
            this.Controls.Add(this.label4);
            this.Controls.Add(this.label3);
            this.Controls.Add(this.label2);
            this.Controls.Add(this.label1);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "Form1";
            this.Text = "Editor urovni hry Obranci";
            ((System.ComponentModel.ISupportInitialize)(this.poradiUrovne)).EndInit();
            this.ResumeLayout(false);
        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}


        private void OnMainButtonClick(object sender, System.EventArgs e)
        {
            Button b = sender as Button;
            int index = (int)b.Tag;
            hlavniBarvy[index]++;
            if (hlavniBarvy[index] >= BarvyKostky.Length)
                hlavniBarvy[index] = 0;
            ((Button)sender).BackColor = BarvyKostky[hlavniBarvy[index]];
            AktualizujBarvyKostky();
        }
        private void OnLevelButtonClick(object sender, EventArgs e)
        {
            Button b = sender as Button;
            int index = (int)b.Tag;
            if ((Form.MouseButtons & MouseButtons.Right) == MouseButtons.Right)
            {
                indexKostkyPostavy = index;
            }
            ukazateleUrovne[index]++;
            if (ukazateleUrovne[index] >= 5)
                ukazateleUrovne[index] = 0;
            AktualizujBarvyKostky();
        }

    }
}
